package com.flyfish.framework.beans.repository;

import com.flyfish.framework.repository.DefaultReactiveRepository;
import com.flyfish.framework.repository.DefaultRepository;
import org.springframework.beans.factory.support.BeanDefinitionRegistry;
import org.springframework.beans.factory.support.BeanNameGenerator;
import org.springframework.core.env.Environment;
import org.springframework.core.io.ResourceLoader;
import org.springframework.core.type.AnnotationMetadata;
import org.springframework.data.mongodb.repository.config.EnableMongoRepositories;
import org.springframework.data.mongodb.repository.config.EnableReactiveMongoRepositories;
import org.springframework.data.mongodb.repository.config.MongoRepositoryConfigurationExtension;
import org.springframework.data.mongodb.repository.config.ReactiveMongoRepositoryConfigurationExtension;

import java.util.ArrayList;
import java.util.List;

/**
 * 注册器组合对象
 *
 * @author wangyu
 */
class RepositoryRegistrarComposite implements RepositoryRegistrar {

    private final List<RepositoryRegistrar> registrars;

    RepositoryRegistrarComposite(AnnotationMetadata metadata) {
        List<RepositoryRegistrar> list = new ArrayList<>();
        if (metadata.hasMetaAnnotation(EnableMongoRepositories.class.getName())) {
            list.add(new SimpleRepositoryRegistrar(EnableMongoRepositories.class, new MongoRepositoryConfigurationExtension(),
                    DefaultRepository.class));
        }
        if (metadata.hasMetaAnnotation(EnableReactiveMongoRepositories.class.getName())) {
            list.add(new SimpleRepositoryRegistrar(EnableReactiveMongoRepositories.class, new ReactiveMongoRepositoryConfigurationExtension(),
                    DefaultReactiveRepository.class));
        }
        this.registrars = list;
    }

    @Override
    public RepositoryRegistrar init(AnnotationMetadata metadata, ResourceLoader resourceLoader, Environment environment,
                                    BeanDefinitionRegistry registry, BeanNameGenerator generator) {
        registrars.forEach(registrar -> registrar.init(metadata, resourceLoader, environment, registry, generator));
        return this;
    }

    @Override
    public void register(Class<?> clazz) {
        if (null == clazz) return;
        registrars.forEach(registrar -> registrar.register(clazz));
    }
}
